#ifndef _sl2_h    // Put these two lines at the top of your file.
#define _sl2_h

#include "sl2cred.h"  //WiFi credentials

//LED pin assignments: GPIO pins safe to use
const byte LED_WiFi = 32; //blue LED on GPIO32 to GND via 3k3
const byte LED_Pulse = 33; //yellow LED to GND - just copies value on intPin
const byte intPin = 13; //interrupt input

//variables to manage LEDs
int WiFiState = 0; //blue LED to show if WiFi connected
int pulseState = 0; //holds value for reflecting input pulse to LED
int oldPulseState = 0; //also with above used in pollInput();

// Control and timing of state machine
int state = 0;  //condition of state machine
unsigned long previousMillis = 0;
unsigned long timerDelay = 100;  //100 to update web page every 30 seconds; 5 gives guru error but seems ok at 10
int repCount = 0;
int repTarget = 6; //events sent only once every 6 sec.
int period = 6; //used in sl2data.checkPeriod to set number of minutes tha have passed.

//pulse debounce
unsigned long tNow, tLast = 0; //used in sl2data:ISR();

//  *** pulse counts  and power calculation; also data for web page (state machine)***
volatile int pCount; // count of pulses in 360sec period
//cWatts, mWatts, pWatts, dayTotal all count * 10
volatile int cWatts; //current value of watts (incrementing over time within period, reset at end)
int pWatts; //previous value, stable, to display on webpage
int mWatts; //current maximum, display on webpage and saved to file at day end
int dayTotalW; //total for day in Wh - need to divide by 1000 for kWh; used & returned in gatherDay();

float dayTotalkWh; //in SPIFFS dayData.log file -  transfer to month record at end of day.
float dTotalkWh; //running total for day - webpage.
float yTotalkWh = 12.675; //total for previous day - webpage. (test data)

char myDateBuf[ 20 ]; //buffer for date to display on webpage - state 3; date dd/mm/yyyy;_hh:mm = 17 char
char webpageDataBuffer[ 12] ; // buffer for power data to display on webpage - states 4 on; max kWh = eg 17.123 = 6 char

String header; // Variable to store the HTTP request
char dayStamp[12], timeStamp[12];
String myDate; //formatted DD/MM/YYYY, HH:MM
//String myDate = "28/06/2021, 06:26"; //formatted DD/MM/YYYY, HH:MM //test data

//  *** for ntp  ***
const char* ntpServer = "pool.ntp.org";
const long  gmtOffset_sec = 0;
const int   daylightOffset_sec = 3600;
struct tm timeinfo;

//  *** for SPIFFS and file handling  ***
char fileName [16];  //stores day end values for month; "/" + 4 digit year + "_" + 2 digit month +".csv" + null = 13 chars
int cDay, cHour, cMin, cSec; //current values -cMin used in checkPeriod
int currMonth = 0, currDay = 0; //tracking newMonth(), newDay()
char dataString[30];  //one line of data per day - in a file for a single month.
char headerString[30] = "Day,total kWh,maxWatts \n";  // = 26 ch
char dayFileName [16] = "/dayData.log";

// *** function prototypes  ***

//in "sl2data"
void configInterrupt(); 
void IRAM_ATTR ISR();  //
void zeroStats();
void checkPeriod();
void newDay();
void newMonth();
void pollInput(); //not used

//in "sl2web"
void buildDate(); // get integer values for date & time, build dayStamp, timeStamp, into myDate
String processor(const String& var); // to replace placeholders in webpage

//in "spiffs"
void appendInt(fs::FS &fs, const char * path, int iData);  //add an ineger value to the /dayData.log file
int gatherDay(fs::FS &fs, const char * path); //sum all the stored values for the day and return the total.
void writeFile(fs::FS &fs, const char * path, const char * message) ;
void appendFile(fs::FS &fs, const char * path, const char * message);
void deleteFile(fs::FS &fs, const char * path);

// in "wifi" - from https://techtutorialsx.com/2019/09/22/esp32-soft-ap-station-disconnected-event/
void initWiFi();
void WiFiStationConnected(WiFiEvent_t event, WiFiEventInfo_t info) ;
void WiFiGotIP(WiFiEvent_t event, WiFiEventInfo_t info);
void WiFiStationDisconnected(WiFiEvent_t event, WiFiEventInfo_t info);


//web page display
const char index_html[] PROGMEM = R"rawliteral(

<!DOCTYPE HTML><html>
<head>
  <title>ESP32 Solar Panel Monitor</title>
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <!-- 
  <link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.7.2/css/all.css" integrity="sha384-fnmOCqbTlWIlj8LyTjo7mOUStjsKC4pOpQbqyi7RrhN7udi9RwhKkMHpvLbHG9Sr" crossorigin="anonymous">
  <link rel="icon" href="data:,">
  -->
  <style>
    html {font-family: Arial; display: inline-block; text-align: center;}
    p {
  font-size: 1.4rem;
  text-align: center;
}
    body {  margin: 0;}
    .topnav { overflow: hidden; background-color: #50B8B4; color: white; font-size: 1rem; }
    .content { padding: 20px; }
    .card { background-color: white; box-shadow: 2px 2px 12px 1px rgba(140,140,140,.5); }
    .cards { max-width: 800px; margin: 0 auto; display: grid; grid-gap: 2rem; grid-template-columns: repeat(auto-fit, minmax(200px, 1fr)); }
    .cards1 { max-width: 800px; margin: 0 auto; display: grid; grid-gap: 2rem; grid-template-columns: repeat(auto-fit, minmax(300px, 1fr)); }
   .reading {
  font-size: 1.6rem;
  color: #004A02;
}
    .reading1 {
  font-size: 1.6rem;
  color: #000EB7;
}
   .reading2 {
  font-size: 1.6rem;
  color: #00820A;
}
  </style>
</head>

<body>
  <div class="topnav">
    <h1>ESP32 Solar Power Monitor</h1>
    <h2>Mk 2.18 Sept 2021</h2>
  </div>
  <div class="content">
    <div class="cards1">
      <div class="card">
        <p>TIME</p><p><span class="reading"><span id="time">%TIME%</span></span></p>
      </div>
    </div><!-- single box -->
    <p>Energy Readings kWh</p>
    <div class="cards">
      <div class="card">
        <p>Today</p><p><span class="reading1"><span id="dtotal">%DTOTAL%</span> kWh</span></p>
      </div>
      <div class="card">
        <p>Yesterday</p><p><span class="reading1"><span id="ytotal">%YTOTAL%</span> kWh</span></p>
      </div>
    </div><!-- 2 box -->
  <p>Power readings - Watts</p>
  <div class="cards">
      <div class="card">
        <p>Now</p><p><span class="reading2"><span id="pwatts">%PWATTS%</span> W</span></p>
      </div>
      <div class="card">
        <p>Maximum</p><p><span class="reading2"><span id="mwatts">%MWATTS%</span> W</span></p>
      </div>
    </div><!-- 2 box -->
  <p>Current count in 6 min period</p>
  <div class="cards1">
      <div class="card">
        <p>Counting</p><p><span class="reading2"><span id="cwatts">%CWATTS%</span> W</span></p>
      </div>
    </div><!-- single box -->
  </div>
  
<script>
if (!!window.EventSource) {
 var source = new EventSource('/events');
 
 source.addEventListener('open', function(e) {
  console.log("Events Connected");
 }, false);
 
 source.addEventListener('error', function(e) {
  if (e.target.readyState != EventSource.OPEN) {
    console.log("Events Disconnected");
  }
 }, false);
 
 source.addEventListener('message', function(e) {
  console.log("message", e.data);
 }, false);
 
 source.addEventListener('time', function(e) {
  console.log("time", e.data);
  document.getElementById("time").innerHTML = e.data;
 }, false);
 
 source.addEventListener('dtotal', function(e) {
  console.log("dtotal", e.data);
  document.getElementById("dtotal").innerHTML = e.data;
 }, false);
 
 source.addEventListener('ytotal', function(e) {
  console.log("ytotal", e.data);
  document.getElementById("ytotal").innerHTML = e.data;
 }, false);
 
 source.addEventListener('pwatts', function(e) {
  console.log("pwatts", e.data);
  document.getElementById("pwatts").innerHTML = e.data;
 }, false);
 
 source.addEventListener('mwatts', function(e) {
  console.log("mwatts", e.data);
  document.getElementById("mwatts").innerHTML = e.data;
 }, false);
 
 source.addEventListener('cwatts', function(e) {
  console.log("cwatts", e.data);
  document.getElementById("cwatts").innerHTML = e.data;
 }, false);
 
}
</script>
</body>
</html>

)rawliteral";


#endif // _sl2_h 
